@@ -8,11 +8,11 @@ if __name__ == '__main__':
   import zebrazoom.code.paths as paths
 
   from zebrazoom.code.vars import getGlobalVariables
-  from zebrazoom.GUIAllPy import PlainApplication, ZebraZoomApp
   globalVariables = getGlobalVariables()
 
   if len(sys.argv) == 1:
 
+    from zebrazoom.GUIAllPy import ZebraZoomApp
     print("The data produced by ZebraZoom can be found in the folder: " + paths.getDefaultZZoutputFolder())
     app = ZebraZoomApp(sys.argv)
     sys.exit(app.exec())
@@ -20,9 +20,9 @@ if __name__ == '__main__':
   else:
 
     if sys.argv[1] == "selectZZoutput" or sys.argv[1] == "--exit":
+      from zebrazoom.GUIAllPy import ZebraZoomApp
       app = ZebraZoomApp(sys.argv)
-    else:
-      app = PlainApplication(sys.argv)
+
     if sys.argv[1] == "selectZZoutput":
 
       print("The data produced by ZebraZoom can be found in the folder: " + paths.getDefaultZZoutputFolder())
@@ -87,7 +87,7 @@ if __name__ == '__main__':
       time.sleep(2)
       print("Launching the tracking, the data produced by ZebraZoom can be found in the folder: " + paths.getDefaultZZoutputFolder())
       __spec__ = "ModuleSpec(name='builtins', loader=<class '_frozen_importlib.BuiltinImporter'>)"
-      mainZZ(path2, videoName, 'avi', configFile, argv2)
+      mainZZ(path2, videoName, 'avi', configFile, argv2, useGUI=False)
 
 
     elif sys.argv[1] == "DL_createMask":
@@ -227,6 +227,15 @@ if __name__ == '__main__':
       videoExt    = sys.argv[3]
       configFile  = sys.argv[4]
       argv        = sys.argv
+
       from zebrazoom.mainZZ import mainZZ
+
+      try:
+        from zebrazoom.GUIAllPy import PlainApplication
+        app = PlainApplication(sys.argv)
+        useGUI = True
+      except ImportError:
+        useGUI = False
+        print("GUI not available")
       __spec__ = "ModuleSpec(name='builtins', loader=<class '_frozen_importlib.BuiltinImporter'>)"
-      mainZZ(pathToVideo, videoName, videoExt, configFile, argv)
+      mainZZ(pathToVideo, videoName, videoExt, configFile, argv, useGUI=useGUI)
