@@ -19,6 +19,7 @@ from PyQt5.QtNetwork import QNetworkAccessManager, QNetworkReply, QNetworkReques
 from PyQt5.QtWidgets import QApplication, QLabel, QWidget, QFileSystemModel, QFrame, QGridLayout, QHeaderView, QPushButton, QSizePolicy, QHBoxLayout, QVBoxLayout, QCheckBox, QScrollArea, QSpinBox, QComboBox, QTreeView, QToolTip
 PYQT6 = False
 
+import zebrazoom
 import zebrazoom.code.paths as paths
 import zebrazoom.code.util as util
 from zebrazoom.code.readValidationVideo import readValidationVideo
@@ -223,6 +224,9 @@ class StartPage(QWidget):
           layout.setColumnStretch(idx, 1)
 
         bottomLayout = QHBoxLayout()
+        self._selectZZoutputFolder = QPushButton("Select output folder")
+        self._selectZZoutputFolder.clicked.connect(controller.askForZZoutputLocation)
+        bottomLayout.addWidget(self._selectZZoutputFolder)
         self._troubleshootBtn = QPushButton("Troubleshoot")
         self._troubleshootBtn.clicked.connect(lambda: controller.show_frame("ChooseVideoToTroubleshootSplitVideo"))
         bottomLayout.addWidget(self._troubleshootBtn)
@@ -281,12 +285,12 @@ class StartPage(QWidget):
             return
         latestVersion = self._reply.url().toString().split('/')[-1]
         self._reply = None
-        if version.parse(latestVersion) <= version.parse(self.controller.version):
-            self._updateStatusLabel.setText("Using ZebraZoom version %s, no updates available." % self.controller.version)
+        if version.parse(latestVersion) <= version.parse(zebrazoom.__version__):
+            self._updateStatusLabel.setText("Using ZebraZoom version %s, no updates available." % zebrazoom.__version__)
             util.apply_style(self._updateStatusLabel, color='green')
             self._updateBtn.hide()
             return
-        self._updateStatusLabel.setText("Using ZebraZoom version %s, updates are available (%s)." % (self.controller.version, latestVersion))
+        self._updateStatusLabel.setText("Using ZebraZoom version %s, updates are available (%s)." % (zebrazoom.__version__, latestVersion))
         util.apply_style(self._updateStatusLabel, color='red')
         self._updateBtn.setText("Update")
         self._updateBtn.show()
@@ -364,7 +368,7 @@ class VideoToAnalyze(QWidget):
         layout = QVBoxLayout()
         layout.addWidget(util.apply_style(QLabel("Choose video.", self), font=controller.title_font), alignment=Qt.AlignmentFlag.AlignCenter)
         layout.addWidget(QLabel("Look for the video you want to analyze.", self), alignment=Qt.AlignmentFlag.AlignCenter)
-        testCheckbox = QCheckBox("Test only on a small number of frames", self)
+        testCheckbox = QCheckBox("Test only on a small number of frames (tracking results will only be saved temporarily)", self)
         layout.addWidget(testCheckbox, alignment=Qt.AlignmentFlag.AlignCenter)
         button = util.apply_style(QPushButton("Choose file", self), background_color=util.LIGHT_YELLOW)
         button.clicked.connect(lambda: controller.chooseVideoToAnalyze(just_extract_checkbox.isChecked(), no_validation_checkbox.isChecked(), chooseFramesCheckbox.isChecked(), testCheckbox.isChecked()))
@@ -372,7 +376,7 @@ class VideoToAnalyze(QWidget):
 
         layout.addWidget(QLabel("", self), alignment=Qt.AlignmentFlag.AlignCenter)
 
-        chooseFramesCheckbox = QCheckBox("Choose the first and the last frames on which the tracking should run", self)
+        chooseFramesCheckbox = QCheckBox("Choose the first and the last frames on which the tracking should run (tracking results will be saved)", self)
         layout.addWidget(chooseFramesCheckbox, alignment=Qt.AlignmentFlag.AlignCenter)
 
         button = util.apply_style(QPushButton("Click here if you prefer to run the tracking from the command line", self), background_color='green')
@@ -451,8 +455,10 @@ class FolderMultipleROIInitialSelect(QWidget):
         layout.addWidget(util.apply_style(QLabel("Choose folder.", self), font=controller.title_font), alignment=Qt.AlignmentFlag.AlignCenter)
         layout.addWidget(QLabel("Select the folder of videos for which you want to define the regions of interest.", self), alignment=Qt.AlignmentFlag.AlignCenter)
         button = util.apply_style(QPushButton("Choose folder", self), background_color=util.LIGHT_YELLOW)
-        button.clicked.connect(controller.chooseFolderForMultipleROIs)
+        button.clicked.connect(lambda: controller.chooseFolderForMultipleROIs(not sameCoordinatesForAllCheckbox.isChecked()))
         layout.addWidget(button, alignment=Qt.AlignmentFlag.AlignCenter)
+        sameCoordinatesForAllCheckbox = QCheckBox("Use the same coordinates for all videos")
+        layout.addWidget(sameCoordinatesForAllCheckbox, alignment=Qt.AlignmentFlag.AlignCenter)
 
         start_page_btn = util.apply_style(QPushButton("Go to the start page", self), background_color=util.LIGHT_CYAN)
         start_page_btn.clicked.connect(lambda: controller.show_frame("StartPage"))
@@ -957,9 +963,11 @@ class ViewParameters(util.CollapsibleSplitter):
                 lengthX  = self.dataRef["wellPositions"][self.numWell()]["lengthX"]
                 lengthY  = self.dataRef["wellPositions"][self.numWell()]["lengthY"]
                 self.a.set_xlim(0, lengthX)
-                self.a.set_ylim(0, lengthY)
+                self.a.set_ylim(lengthY, 0)
 
               self.a.plot(headX, headY)
+              if self.graphScaling:
+                self.a.set_ylim(self.a.get_ylim()[::-1])
         else:
             tailAngleSmoothed = [i for i in range(0,1)]
             self.a.plot([i for i in range(0,len(tailAngleSmoothed))],tailAngleSmoothed)
@@ -1099,12 +1107,14 @@ class ViewParameters(util.CollapsibleSplitter):
         headXFinal = []
         headYFinal = []
         for numMouv in range(0, len(dataRef["wellPoissMouv"][numWell][numPoiss])):
-          headXFinal = headXFinal + dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["HeadX"].copy()
-          headYFinal = headYFinal + dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["HeadY"].copy()
+          headXFinal.extend(dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["HeadX"])
+          headYFinal.extend(dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["HeadY"])
         plt.plot(headXFinal, headYFinal)
         if not(graphScaling):
           plt.xlim(0, dataRef["wellPositions"][numWell]["lengthX"])
-          plt.ylim(0, dataRef["wellPositions"][numWell]["lengthY"])
+          plt.ylim(dataRef["wellPositions"][numWell]["lengthY"], 0)
+        else:
+          plt.ylim(plt.ylim()[::-1])
         plt.show()
 
     def printNextResults(self):
