@@ -3,10 +3,10 @@ import cv2
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
 import math
 import json
-import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
 import random
 import os
 
+
 def createValidationVideo(videoPath, superStruct, hyperparameters):
 
   if (hyperparameters["freqAlgoPosFollow"] != 0):
@@ -143,6 +143,9 @@ def createValidationVideo(videoPath, superStruct, hyperparameters):
               dataToPlot["y"]       = leftEyeY
               dataToPlot["size"]    = 1
               dataToPlot["Heading"] = leftEyeAngle
+              dataToPlot["headingColor"] = (0, 255, 0)
+              dataToPlot["headingWidth"]    = 2 # hyperparameters["eyeTrackingHeadEmbeddedWidth"]
+              dataToPlot["headingHalfDiam"] = hyperparameters["eyeTrackingHeadEmbeddedHalfDiameter"]
               dataToPlot["numMouv"] = k+1
               dataToPlot["numWell"] = i
               dataToPlot["red"]     = 255
@@ -163,6 +166,9 @@ def createValidationVideo(videoPath, superStruct, hyperparameters):
               dataToPlot["y"]       = rightEyeY
               dataToPlot["size"]    = 1
               dataToPlot["Heading"] = rightEyeAngle
+              dataToPlot["headingColor"] = (0, 255, 0)
+              dataToPlot["headingWidth"]    = 2 # hyperparameters["eyeTrackingHeadEmbeddedWidth"]
+              dataToPlot["headingHalfDiam"] = hyperparameters["eyeTrackingHeadEmbeddedHalfDiameter"]
               # dataToPlot["numMouv"] = k+1
               # dataToPlot["numWell"] = i
               dataToPlot["red"]     = 0
@@ -200,17 +206,19 @@ def createValidationVideo(videoPath, superStruct, hyperparameters):
           frame = cv2.resize(frame, (int(frame_width * hyperparameters["reduceImageResolutionPercentage"]), int(frame_height * hyperparameters["reduceImageResolutionPercentage"])), interpolation = cv2.INTER_AREA)
         if hyperparameters["outputValidationVideoContrastImprovement"]:
           frame = 255 - frame
-          frame  = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
-          quartileChose = hyperparameters["outputValidationVideoContrastImprovementQuartile"]
-          lowVal  = int(np.quantile(frame, quartileChose))
-          highVal = int(np.quantile(frame, 1 - quartileChose))
-          frame[frame < lowVal]  = lowVal
+          frameIsBGR = len(frame.shape) == 3
+          if frameIsBGR:
+            frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
+          lowVal = int(np.quantile(frame, hyperparameters["outputValidationVideoContrastImprovementQuartile"]))
+          highVal = int(np.quantile(frame, 1 - hyperparameters["outputValidationVideoContrastImprovementQuartile"]))
+          frame[frame < lowVal] = lowVal
           frame[frame > highVal] = highVal
           frame = frame - lowVal
-          mult  = np.max(frame)
-          frame = frame * (255/mult)
+          mult = np.max(frame)
+          frame = frame * (255 / mult)
           frame = frame.astype('uint8')
-          frame  = cv2.cvtColor(frame, cv2.COLOR_GRAY2BGR)
+          if frameIsBGR:
+            frame = cv2.cvtColor(frame, cv2.COLOR_GRAY2BGR)
         
         for i in range(0, len(infoFrame[l])):
           x = infoFrame[l][i]["x"]
@@ -221,11 +229,14 @@ def createValidationVideo(videoPath, superStruct, hyperparameters):
           blue  = infoFrame[l][i]["blue"]
           if (infoFrame[l][i]["Heading"] != -10):
             heading = infoFrame[l][i]["Heading"]
+            headingColor = infoFrame[l][i]["headingColor"] if "headingColor" in infoFrame[l][i] else (255,0,0)
+            headingWidth = infoFrame[l][i]["headingWidth"] if "headingWidth" in infoFrame[l][i] else trackingPointSizeDisplay
+            headingHalfDiam = infoFrame[l][i]["headingHalfDiam"] if "headingHalfDiam" in infoFrame[l][i] else 20
             if hyperparameters["validationVideoPlotHeading"]:
               if hyperparameters["debugValidationVideoHeading"] == 0:
-                cv2.line(frame,(int(x),int(y)),(int(x+20*math.cos(heading)),int(y+20*math.sin(heading))), (255,0,0), trackingPointSizeDisplay)
+                cv2.line(frame,(int(x),int(y)),(int(x+headingHalfDiam*math.cos(heading)),int(y+headingHalfDiam*math.sin(heading))), headingColor, headingWidth)
               else:
-                cv2.line(frame,(int(x),int(y)),(int(x-250*math.cos(heading)),int(y-250*math.sin(heading))), (255,0,0), trackingPointSizeDisplay)
+                cv2.line(frame,(int(x),int(y)),(int(x-250*math.cos(heading)),int(y-250*math.sin(heading))), headingColor, headingWidth)
             
             # if ("numMouv" in infoFrame[l][i]) and ("numWell" in infoFrame[l][i]):
               # numMouv = infoFrame[l][i]["numMouv"]
@@ -249,6 +260,8 @@ def createValidationVideo(videoPath, superStruct, hyperparameters):
   if (hyperparameters["freqAlgoPosFollow"] != 0):
     print("Validation video created")
   if hyperparameters["popUpAlgoFollow"]:
+    import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
+
     popUpAlgoFollow.prepend("Create Validation Video")
 
   return infoFrame
