@@ -5,13 +5,12 @@ import math
 import json
 import os
 import sys
-import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
 import pandas as pd
 from scipy.interpolate import UnivariateSpline
 
 import numpy as np
-from filterpy.kalman import KalmanFilter
-from filterpy.common import Q_discrete_white_noise
+# from filterpy.kalman import KalmanFilter
+# from filterpy.common import Q_discrete_white_noise
 
 from zebrazoom.code.detectMovementWithRawVideo import detectMovementWithRawVideo
 from zebrazoom.code.trackingFolder.tailTrackingFunctionsFolder.getTailTipManual import getHeadPositionByFileSaved, getTailTipByFileSaved
@@ -56,6 +55,7 @@ def calculateTailAngle(angle1, angle2):
   return output
   
 def smoothAllTailAngles(allAngles, hyperparameters, start, end):
+  # The first angle is removed here because it corresponds to the angle between two same point (the center of the head)
   tailangles_arr = np.transpose(allAngles[start:end+1, 1:len(allAngles)])
   tailangles_arr_smoothed = np.zeros((0, len(tailangles_arr[0])))
   for angle_raw in tailangles_arr:
@@ -98,6 +98,7 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
   trackingEyesAllAnimals     = trackingData[2]
   headPositionFirstFrame     = trackingData[3]
   tailTipFirstFrame          = trackingData[4]
+  auDessusPerAnimal          = trackingData[5] if len(trackingData) == 6 else 0
   
   data = []
   
@@ -154,6 +155,11 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       elif hyperparameters["headingCalculationMethod"] == "simplyFromPreviousCalculations":
         heading[i] = trackingHeading[i]
         heading[i] = (heading[i] + math.pi) % (2*math.pi)
+      elif hyperparameters["headingCalculationMethod"] == "fromPreviousCalculationsAndAdjustWithPreviousFrame":
+        heading[i] = trackingHeading[i]
+        heading[i] = (heading[i] + math.pi) % (2*math.pi)
+        if i != 0 and distBetweenThetas(heading[i-1], ((heading[i] + math.pi) % (2*math.pi))) > distBetweenThetas(heading[i-1], heading[i]):
+          heading[i] = (heading[i] + math.pi) % (2*math.pi)
       else: # calculatedWithHead : THIS IS THE DEFAULT
         if not(math.isnan(trackingHeading[i])):
           heading[i] = trackingHeading[i]
@@ -182,54 +188,59 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       auDessus        = np.zeros((nbFrames, 1))
       for i in range(0,nbFrames):
         auDessus[i] = 1
-    else:
-      if hyperparameters["boutEdgesWhereZeros"] == 1:
-        auDessus        = np.zeros((nbFrames, 1))
-        for i in range(0,nbFrames):
-          if ((head[i][0] == 0) and (head[i][1] == 0)):
-            auDessus[i] = 0
-          else:
-            auDessus[i] = 1
-      else:
-        if hyperparameters["thresForDetectMovementWithRawVideo"] == 0:
-          
-          # Calculating angle median
-          angle2 = np.transpose(angle)
-          angle2 = angle2[0]
-          rolling_window = hyperparameters["tailAngleMedianFilter"]
-          if rolling_window > 0:
-            shift = int(-rolling_window / 2)
-            angle_median = np.array(pd.Series(angle2).rolling(rolling_window).median())
-            angle_median = np.roll(angle_median, shift)
-            for ii in range(0, rolling_window):
-              angle_median[ii] = angle2[ii]
-            for ii in range(len(angle_median)-rolling_window,len(angle_median)):
-              angle_median[ii] = angle2[ii]
-          else:
-            angle_median = angle2
-          # Calculating angle variation to detect movement
-          angleVariation  = np.zeros((nbFrames, 1))
-          auDessus        = np.zeros((nbFrames, 1))
-          for i in range(0,nbFrames):
-            if (hyperparameters["freqAlgoPosFollow"] != 0) and (i % (hyperparameters["freqAlgoPosFollow"]) == 0):
-              print("Extract Param Middle: wellNumber:",wellNumber," ; frame:",i)
-            min = 10000
-            max = -10000
-            # for j in range(i-hyperparameters["windowForBoutDetectWithAngle"], i+1):
-            for j in range(i-int(hyperparameters["windowForBoutDetectWithAngle"]/2), i+int(hyperparameters["windowForBoutDetectWithAngle"]/2)):
-              if (j >= 0) and (j<nbFrames):
-                if angle_median[j] > max:
-                  max = angle_median[j]
-                if angle_median[j] < min:
-                  min = angle_median[j]
-            angleVariation[i] = max - min
-            if debug:
-              print(i,angleVariation[i])
-            if angleVariation[i] > thresAngleBoutDetect:
-              auDessus[i] = 1
-          
+    elif hyperparameters["boutEdgesWhereZeros"] == 1:
+      auDessus        = np.zeros((nbFrames, 1))
+      for i in range(0,nbFrames):
+        if ((head[i][0] == 0) and (head[i][1] == 0)):
+          auDessus[i] = 0
         else:
-          auDessus = detectMovementWithRawVideo(hyperparameters, videoPath, background, wellNumber, wellPositions, head, headPositionFirstFrame, tailTipFirstFrame)
+          auDessus[i] = 1
+    elif type(auDessusPerAnimal) != int:
+
+      auDessus = auDessusPerAnimal[animalId]
+    elif hyperparameters["coordinatesOnlyBoutDetection"]:
+      frameDistance = hyperparameters["frameGapComparision"]
+      auDessus = [int(math.sqrt(sum((head[i+frameDistance] - coords) ** 2)) >= hyperparameters["coordinatesOnlyBoutDetectionMinDist"]) for i, coords in enumerate(head[:-frameDistance])]
+      auDessus.extend([0] * frameDistance)
+    elif hyperparameters["thresForDetectMovementWithRawVideo"] == 0:
+
+      # Calculating angle median
+      angle2 = np.transpose(angle)
+      angle2 = angle2[0]
+      rolling_window = hyperparameters["tailAngleMedianFilter"]
+      if rolling_window > 0:
+        shift = int(-rolling_window / 2)
+        angle_median = np.array(pd.Series(angle2).rolling(rolling_window).median())
+        angle_median = np.roll(angle_median, shift)
+        for ii in range(0, rolling_window):
+          angle_median[ii] = angle2[ii]
+        for ii in range(len(angle_median)-rolling_window,len(angle_median)):
+          angle_median[ii] = angle2[ii]
+      else:
+        angle_median = angle2
+      # Calculating angle variation to detect movement
+      angleVariation  = np.zeros((nbFrames, 1))
+      auDessus        = np.zeros((nbFrames, 1))
+      for i in range(0,nbFrames):
+        if (hyperparameters["freqAlgoPosFollow"] != 0) and (i % (hyperparameters["freqAlgoPosFollow"]) == 0):
+          print("Extract Param Middle: wellNumber:",wellNumber," ; frame:",i)
+        min = 10000
+        max = -10000
+        # for j in range(i-hyperparameters["windowForBoutDetectWithAngle"], i+1):
+        for j in range(i-int(hyperparameters["windowForBoutDetectWithAngle"]/2), i+int(hyperparameters["windowForBoutDetectWithAngle"]/2)):
+          if (j >= 0) and (j<nbFrames):
+            if angle_median[j] > max:
+              max = angle_median[j]
+            if angle_median[j] < min:
+              min = angle_median[j]
+        angleVariation[i] = max - min
+        if debug:
+          print(i,angleVariation[i])
+        if angleVariation[i] > thresAngleBoutDetect:
+          auDessus[i] = 1
+
+    else:
+      auDessus = detectMovementWithRawVideo(hyperparameters, videoPath, background, wellNumber, wellPositions, head, headPositionFirstFrame, tailTipFirstFrame)
     
     if (hyperparameters["freqAlgoPosFollow"] != 0):
       print("Extract Param Middle: wellNumber:",wellNumber," ; frame:",i)
@@ -268,7 +279,7 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       if (position == 1) and ((auDessus2[i] == 0) or (i == nbFrames-1)):
         curBout[0][2] = i-1
         position = 0
-        if curBout[0][2] - curBout[0][1] >= hyperparameters["detectBoutMinNbFrames"] or (hyperparameters["noChecksForBoutSelectionInExtractParams"] and curBout[0][2] - curBout[0][1] >= 1):
+        if curBout[0][2] - curBout[0][1] >= hyperparameters["detectBoutMinNbFrames"] or (hyperparameters["noChecksForBoutSelectionInExtractParams"] and curBout[0][2] - curBout[0][1] >= hyperparameters["boutsMinNbFrames"]):
           debMouv = int(curBout[0][1])
           endMouv = int(curBout[0][2])
           dist = math.sqrt( (head[debMouv,0]-head[endMouv,0])**2 + (head[debMouv,1]-head[endMouv,1])**2 )
@@ -294,7 +305,10 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       pos = finMouv
       while int(auDessus[pos]) == 0:
         pos = pos - 1
-      bouts[numBout][2] = pos
+      bouts[numBout][2] = pos    
+      if hyperparameters["addOneFrameAtTheEndForBoutDetection"]:
+        if bouts[numBout][2] + 1 < nbFrames:
+          bouts[numBout][2] = bouts[numBout][2] + 1
     
     for i in range(0,len(bouts)):
     
@@ -319,9 +333,8 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
         item["rightEyeAngle"] = trackingEyesAllAnimals[animalId, start:end+1, 6].tolist()
         item["rightEyeArea"]  = trackingEyesAllAnimals[animalId, start:end+1, 7].tolist()
       
-      [tailangles_arr, tailangles_arr_smoothed] = smoothAllTailAngles(allAngles, hyperparameters, start, end)
-      
       if hyperparameters["calculateAllTailAngles"]:
+        [tailangles_arr, tailangles_arr_smoothed] = smoothAllTailAngles(allAngles, hyperparameters, start, end)
         item["allTailAngles"]         = tailangles_arr.tolist()
         item["allTailAnglesSmoothed"] = tailangles_arr_smoothed.tolist()
       
@@ -338,6 +351,7 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
   
   print("Parameters extracted for well",wellNumber)
   if hyperparameters["popUpAlgoFollow"]:
+    import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
     popUpAlgoFollow.prepend("Parameters extracted for well " + str(wellNumber))
   
   return data
