@@ -1,18 +1,8 @@
 import numpy as np
 import cv2
-import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
 from zebrazoom.code.preprocessImage import preprocessImage, preprocessBackgroundImage
-from zebrazoom.code.resizeImageTooLarge import resizeImageTooLarge
-import zebrazoom.code.util as util
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
 
-try:
-  from PyQt6.QtCore import Qt
-  from PyQt6.QtWidgets import QApplication, QLabel, QVBoxLayout
-except ImportError:
-  from PyQt5.QtCore import Qt
-  from PyQt5.QtWidgets import QApplication, QLabel, QVBoxLayout
-
 
 def getBackground(videoPath, hyperparameters):
   
@@ -131,27 +121,23 @@ def getBackground(videoPath, hyperparameters):
   if hyperparameters["setBackgroundToImageMedian"]:
     back[:, :] = np.median(back)
   
-  if (debugExtractBack):
-    if QApplication.instance() is None:
-      back2 = back.copy()
-      [back2, getRealValueCoefX, getRealValueCoefY, horizontal, vertical] = resizeImageTooLarge(back2)
-      cv2.imshow('Background extracted: Click on any key to proceed!', back2)
-      if hyperparameters["exitAfterBackgroundExtraction"]:
-        cv2.waitKey(3000)
-      else:
-        cv2.waitKey(0)
-      cv2.destroyAllWindows()
-    else:
-      label = QLabel()
-      label.setMinimumSize(1, 1)
-      layout = QVBoxLayout()
-      layout.addWidget(label, alignment=Qt.AlignmentFlag.AlignCenter)
-      timeout = 3000 if hyperparameters["exitAfterBackgroundExtraction"] else None
-      util.showDialog(layout, title="Background Extracted", labelInfo=(back, label), timeout=timeout)
+  if debugExtractBack:
+    from PyQt5.QtCore import Qt
+    from PyQt5.QtWidgets import QLabel, QVBoxLayout
+
+    import zebrazoom.code.util as util
+
+    label = QLabel()
+    label.setMinimumSize(1, 1)
+    layout = QVBoxLayout()
+    layout.addWidget(label, alignment=Qt.AlignmentFlag.AlignCenter)
+    timeout = 3000 if hyperparameters["exitAfterBackgroundExtraction"] else None
+    util.showDialog(layout, title="Background Extracted", labelInfo=(back, label), timeout=timeout)
   cap.release()
   
   print("Background Extracted")
   if hyperparameters["popUpAlgoFollow"]:
+    import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
     popUpAlgoFollow.prepend("Background Extracted")
   
   return back
