@@ -3,6 +3,7 @@ import json
 import cv2
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
 import os
+import zebrazoom.code.paths as paths
 from zebrazoom.code.vars import getGlobalVariables
 globalVariables = getGlobalVariables()
 
@@ -26,7 +27,7 @@ def getConfig(config, variableName, videoPath):
     if variableName in configDefault:
       
       if variableName == "outputFolder":
-        return os.path.join(cur_dir_path2, "ZZoutput")
+        return paths.getDefaultZZoutputFolder()
       else:
         return configDefault[variableName]
         
@@ -204,6 +205,10 @@ def getHyperparameters(configFile, videoName, videoPath, argv):
   
   hyperparameters["noBoutsDetection"]    = getConfig(config, "noBoutsDetection", videoPath)
   
+  hyperparameters["coordinatesOnlyBoutDetection"] = getConfig(config, "coordinatesOnlyBoutDetection", videoPath)
+
+  hyperparameters["coordinatesOnlyBoutDetectionMinDist"] = getConfig(config, "coordinatesOnlyBoutDetectionMinDist", videoPath)
+
   hyperparameters["setPixDiffBoutDetectParameters"] = getConfig(config, "setPixDiffBoutDetectParameters", videoPath)
   
   hyperparameters["midlineIsInBlobTrackingOptimization"] = getConfig(config, "midlineIsInBlobTrackingOptimization", videoPath)
@@ -263,6 +268,7 @@ def getHyperparameters(configFile, videoName, videoPath, argv):
   hyperparameters["adjustDetectMovWithRawVideo"] = getConfig(config, "adjustDetectMovWithRawVideo", videoPath)
   
   hyperparameters["adjustHeadEmbededTracking"] = getConfig(config, "adjustHeadEmbededTracking", videoPath)
+  hyperparameters["adjustHeadEmbeddedEyeTracking"] = getConfig(config, "adjustHeadEmbeddedEyeTracking", videoPath)
   hyperparameters["adjustFreelySwimTracking"] = getConfig(config, "adjustFreelySwimTracking", videoPath)
   hyperparameters["adjustFreelySwimTrackingAutomaticParameters"] = getConfig(config, "adjustFreelySwimTrackingAutomaticParameters", videoPath)
   
@@ -438,6 +444,20 @@ def getHyperparameters(configFile, videoName, videoPath, argv):
   
   hyperparameters["colorMapCurvature"]  = getConfig(config, "colorMapCurvature", videoPath)
   
+  hyperparameters["eyeTrackingHeadEmbeddedWidth"]        = getConfig(config, "eyeTrackingHeadEmbeddedWidth", videoPath)
+  hyperparameters["eyeTrackingHeadEmbeddedWidthLeft"]    = getConfig(config, "eyeTrackingHeadEmbeddedWidthLeft", videoPath)
+  hyperparameters["eyeTrackingHeadEmbeddedWidthRight"]   = getConfig(config, "eyeTrackingHeadEmbeddedWidthRight", videoPath)
+  hyperparameters["eyeTrackingHeadEmbeddedHalfDiameter"] = getConfig(config, "eyeTrackingHeadEmbeddedHalfDiameter", videoPath)
+  
+  hyperparameters["eyeTrackingHeadEmbeddedWithSegment"]  = getConfig(config, "eyeTrackingHeadEmbeddedWithSegment", videoPath)
+  hyperparameters["eyeTrackingHeadEmbeddedWithEllipse"]  = getConfig(config, "eyeTrackingHeadEmbeddedWithEllipse", videoPath)
+  hyperparameters["eyeFilterKernelSize"] = getConfig(config, "eyeFilterKernelSize", videoPath)
+  
+  hyperparameters["noPreProcessingOfImageForBoutDetection"] = getConfig(config, "noPreProcessingOfImageForBoutDetection", videoPath)
+  
+  hyperparameters["invertColorsForHeadEmbeddedEyeTracking"] = getConfig(config, "invertColorsForHeadEmbeddedEyeTracking", videoPath)
+  hyperparameters["improveContrastForEyeDetectionOfHeadEmbedded"] = getConfig(config, "improveContrastForEyeDetectionOfHeadEmbedded", videoPath)
+  
   if hyperparameters["tailAnglesHeatMap"]:
     hyperparameters["calculateAllTailAngles"] = 1
   
