@@ -2,7 +2,6 @@ import h5py
 import numpy as np
 import cv2
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
-from zebrazoom.code.popUpAlgoFollow import prepend
 import math
 import os
 import queue
@@ -13,13 +12,13 @@ from zebrazoom.code.trackingFolder.debugTracking import debugTracking
 from zebrazoom.code.trackingFolder.tailTracking import tailTracking
 from zebrazoom.code.trackingFolder.blackFramesDetection import getThresForBlackFrame, savingBlackFrames
 from zebrazoom.code.trackingFolder.tailTrackingFunctionsFolder.getTailTipManual import getHeadPositionByFileSaved, findTailTipByUserInput, getTailTipByFileSaved, findHeadPositionByUserInput, getAccentuateFrameForManualPointSelect
-from zebrazoom.code.trackingFolder.eyeTracking.eyeTracking import eyeTracking
+from zebrazoom.code.trackingFolder.eyeTracking.eyeTracking import eyeTracking, eyeTrackingHeadEmbedded
 from zebrazoom.code.trackingFolder.postProcessMultipleTrajectories import postProcessMultipleTrajectories
 
 from zebrazoom.code.getImage.headEmbededFrame import headEmbededFrame
 from zebrazoom.code.getImage.headEmbededFrameBackExtract import headEmbededFrameBackExtract
 
-from zebrazoom.code.adjustHyperparameters import initializeAdjustHyperparametersWindows, adjustHyperparameters, getHeadEmbededTrackingParamsForHyperParamAdjusts, getFreelySwimTrackingParamsForHyperParamAdjusts, getFreelySwimTrackingAutoParamsForHyperParamAdjusts
+from zebrazoom.code.adjustHyperparameters import adjustHeadEmbededTrackingParams, adjustFreelySwimTrackingParams, adjustFreelySwimTrackingAutoParams
 from zebrazoom.code.trackingFolder.tailTrackingFunctionsFolder.headEmbededTailTracking import adjustHeadEmbededHyperparameters
 
 from zebrazoom.code.trackingFolder.tailTrackingFunctionsFolder.headEmbededTailTracking import headEmbededTailTrackFindMaxDepth
@@ -28,6 +27,7 @@ from zebrazoom.code.trackingFolder.tailTrackingFunctionsFolder.centerOfMassTailT
 
 from zebrazoom.code.trackingFolder.trackingFunctions import addBlackLineToImgSetParameters
 
+
 def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters, videoName, dlModel=0):
   
   if hyperparameters["trackingDL"]:
@@ -91,16 +91,32 @@ def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters,
     # Getting images
     
     if hyperparameters["headEmbededRemoveBack"] == 0 and hyperparameters["headEmbededAutoSet_BackgroundExtractionOption"] == 0:
-      [frame, thresh1] = headEmbededFrame(videoPath, firstFrame, hyperparameters)
+      [frame, thresh1] = headEmbededFrame(videoPath, firstFrame, wellNumber, wellPositions, hyperparameters)
     else:
       hyperparameters["headEmbededRemoveBack"] = 1
       hyperparameters["minPixelDiffForBackExtract"] = hyperparameters["headEmbededAutoSet_BackgroundExtractionOption"]
-      [frame, thresh1] = headEmbededFrameBackExtract(videoPath, background, hyperparameters, firstFrame)
+      [frame, thresh1] = headEmbededFrameBackExtract(videoPath, background, hyperparameters, firstFrame, wellNumber, wellPositions)
     
     # Setting hyperparameters in order to add line on image
     if hyperparameters["addBlackLineToImg_Width"]:
       hyperparameters = addBlackLineToImgSetParameters(hyperparameters, frame, videoName)
     
+    # (x, y) coordinates for both eyes for head embedded fish eye tracking
+    if hyperparameters["eyeTracking"] and hyperparameters["headEmbeded"] == 1:
+      forEye = getAccentuateFrameForManualPointSelect(frame, hyperparameters)
+      if True:
+        from PyQt5.QtWidgets import QApplication
+
+        import zebrazoom.code.util as util
+
+        leftEyeCoordinate = list(util.getPoint(np.uint8(forEye * 255), "Click on the center of the left eye", zoomable=True, dialog=not hasattr(QApplication.instance(), 'window')))
+        rightEyeCoordinate = list(util.getPoint(np.uint8(forEye * 255), "Click on the center of the right eye", zoomable=True, dialog=not hasattr(QApplication.instance(), 'window')))
+      else:
+        leftEyeCoordinate  = [261, 201] # [267, 198] # [210, 105]
+        rightEyeCoordinate = [285, 157] # [290, 151] # [236, 72]
+      print("leftEyeCoordinate:", leftEyeCoordinate)
+      print("rightEyeCoordinate:", rightEyeCoordinate)
+    
     # if hyperparameters["invertBlackWhiteOnImages"]:
       # frame   = 255 - frame
     
@@ -114,7 +130,7 @@ def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters,
     else:
       if hyperparameters["findHeadPositionByUserInput"]:
         frameForManualPointSelection = getAccentuateFrameForManualPointSelect(frame, hyperparameters)
-        headPositionFirstFrame = findHeadPositionByUserInput(frameForManualPointSelection, firstFrame, videoPath)
+        headPositionFirstFrame = findHeadPositionByUserInput(frameForManualPointSelection, firstFrame, videoPath, hyperparameters, wellNumber, wellPositions)
       else:
         [frame, gray, thresh1, blur, thresh2, frame2, initialCurFrame, back, xHead, yHead] = getImages(hyperparameters, cap, videoPath, firstFrame, background, wellNumber, wellPositions)
         cap.set(1, firstFrame)
@@ -123,7 +139,7 @@ def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters,
       tailTipFirstFrame  = getTailTipByFileSaved(hyperparameters,videoPath)
     else:
       frameForManualPointSelection = getAccentuateFrameForManualPointSelect(frame, hyperparameters)
-      tailTipFirstFrame  = findTailTipByUserInput(frameForManualPointSelection, firstFrame, videoPath, hyperparameters)
+      tailTipFirstFrame  = findTailTipByUserInput(frameForManualPointSelection, firstFrame, videoPath, hyperparameters, wellNumber, wellPositions)
     if hyperparameters["automaticallySetSomeOfTheHeadEmbededHyperparameters"] == 1:
       hyperparameters = adjustHeadEmbededHyperparameters(hyperparameters, frame, headPositionFirstFrame, tailTipFirstFrame)
     # Getting max depth
@@ -137,9 +153,8 @@ def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters,
       else:
         maxDepth = centerOfMassTailTrackFindMaxDepth(headPositionFirstFrame,nbTailPoints,firstFrame,headPositionFirstFrame[0],headPositionFirstFrame[1],thresh1,frame,hyperparameters,oppHeading,tailTipFirstFrame)
   
-  if hyperparameters["adjustHeadEmbededTracking"] == 1 or hyperparameters["adjustFreelySwimTracking"] == 1 or hyperparameters["adjustFreelySwimTrackingAutomaticParameters"] == 1:
-    initializeAdjustHyperparametersWindows("Tracking")
-  organizationTabCur = []
+  if hyperparameters["adjustHeadEmbededTracking"] == 1 or hyperparameters["adjustFreelySwimTracking"] == 1 or hyperparameters["adjustFreelySwimTrackingAutomaticParameters"] == 1 or hyperparameters["adjustHeadEmbeddedEyeTracking"]:
+    widgets = None
   
   # Performing the tracking on each frame
   i = firstFrame
@@ -150,6 +165,7 @@ def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters,
     if (hyperparameters["freqAlgoPosFollow"] != 0) and (i % hyperparameters["freqAlgoPosFollow"] == 0):
       print("Tracking: wellNumber:",wellNumber," ; frame:",i)
       if hyperparameters["popUpAlgoFollow"]:
+        from zebrazoom.code.popUpAlgoFollow import prepend
         prepend("Tracking: wellNumber:" + str(wellNumber) + " ; frame:" + str(i))
     if hyperparameters["debugTracking"]:
       print("frame:",i)
@@ -163,7 +179,17 @@ def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters,
         [trackingHeadTailAllAnimals, trackingHeadingAllAnimals] = tailTracking(animalId, i, firstFrame, videoPath, frame, hyperparameters, thresh1, nbTailPoints, threshForBlackFrames, thetaDiffAccept, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, lastFirstTheta, maxDepth, tailTipFirstFrame, initialCurFrame, back, wellNumber, xHead, yHead)
     # Eye tracking for frame i
     if hyperparameters["eyeTracking"]:
-      trackingEyesAllAnimals = eyeTracking(animalId, i, firstFrame, frame, hyperparameters, thresh1, trackingHeadingAllAnimals, trackingHeadTailAllAnimals, trackingEyesAllAnimals)
+      if hyperparameters["headEmbeded"] == 1:
+        if hyperparameters["adjustHeadEmbeddedEyeTracking"]:
+          i, widgets = eyeTrackingHeadEmbedded(animalId, i, firstFrame, frame, hyperparameters, thresh1, trackingHeadingAllAnimals, trackingHeadTailAllAnimals, trackingEyesAllAnimals, leftEyeCoordinate, rightEyeCoordinate, widgets=widgets)
+          if not hyperparameters["eyeFilterKernelSize"] % 2:
+            hyperparameters["eyeFilterKernelSize"] -= 1
+          continue
+        else:
+          trackingEyesAllAnimals = eyeTrackingHeadEmbedded(animalId, i, firstFrame, frame, hyperparameters, thresh1, trackingHeadingAllAnimals, trackingHeadTailAllAnimals, trackingEyesAllAnimals, leftEyeCoordinate, rightEyeCoordinate)
+      else:
+        trackingEyesAllAnimals = eyeTracking(animalId, i, firstFrame, frame, hyperparameters, thresh1, trackingHeadingAllAnimals, trackingHeadTailAllAnimals, trackingEyesAllAnimals)
+    
     # Debug functions
     if hyperparameters["nbAnimalsPerWell"] > 1 or hyperparameters["forceBlobMethodForHeadTracking"] or hyperparameters["headEmbeded"] == 1 or hyperparameters["fixedHeadPositionX"] != -1:
       debugTracking(nbTailPoints, i, firstFrame, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, frame, hyperparameters)
@@ -215,16 +241,10 @@ def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters,
             trackingHeadTailAllAnimals[animalId][i-firstFrame][j][1] = trackingHeadTailAllAnimals[animalId][i-firstFrame][j][1] + yHead
     
     if hyperparameters["adjustHeadEmbededTracking"] == 1:
-      [hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTab] = getHeadEmbededTrackingParamsForHyperParamAdjusts(nbTailPoints, i, firstFrame, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, frame, frame2, hyperparameters)
-      if len(organizationTabCur) == 0:
-        organizationTabCur = organizationTab
-      [i, hyperparameters, organizationTabCur] = adjustHyperparameters(i, hyperparameters, hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTabCur)
+      i, widgets = adjustHeadEmbededTrackingParams(nbTailPoints, i, firstFrame, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, frame, frame2, hyperparameters, widgets)
       hyperparameters = adjustHeadEmbededHyperparameters(hyperparameters, frame, headPositionFirstFrame, tailTipFirstFrame)
     elif hyperparameters["adjustFreelySwimTracking"] == 1:
-      [hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTab] = getFreelySwimTrackingParamsForHyperParamAdjusts(nbTailPoints, i, firstFrame, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, frame, frame2, hyperparameters)
-      if len(organizationTabCur) == 0:
-        organizationTabCur = organizationTab
-      [i, hyperparameters, organizationTabCur] = adjustHyperparameters(i, hyperparameters, hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTabCur)
+      i, widgets = adjustFreelySwimTrackingParams(nbTailPoints, i, firstFrame, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, frame, frame2, hyperparameters, widgets)
     elif hyperparameters["adjustFreelySwimTrackingAutomaticParameters"] == 1:
       # Preparing image to show
       if hyperparameters["recalculateForegroundImageBasedOnBodyArea"] == 1:
@@ -240,10 +260,7 @@ def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters,
       curFrame[putToWhite] = 255
       ret, frame2 = cv2.threshold(curFrame, hyperparameters["thresholdForBlobImg"], 255, cv2.THRESH_BINARY)
       # Showing current image and waiting for next parameter/frame change
-      [hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTab] = getFreelySwimTrackingAutoParamsForHyperParamAdjusts(nbTailPoints, i, firstFrame, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, frame, frame2, hyperparameters)
-      if len(organizationTabCur) == 0:
-        organizationTabCur = organizationTab
-      [i, hyperparameters, organizationTabCur] = adjustHyperparameters(i, hyperparameters, hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTabCur)
+      i, widgets = adjustFreelySwimTrackingAutoParams(nbTailPoints, i, firstFrame, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, frame, frame2, hyperparameters, widgets)
       # Puts hyperparameters values to accepted values
       hyperparameters["recalculateForegroundImageBasedOnBodyArea"] = 0 if hyperparameters["recalculateForegroundImageBasedOnBodyArea"] < 0.5 else 1
       if hyperparameters["adjustMinPixelDiffForBackExtract_nbBlackPixelsMax"] < 0:
@@ -265,6 +282,7 @@ def tracking(videoPath, background, wellNumber, wellPositions, hyperparameters,
   
   print("Tracking done for well", wellNumber)
   if hyperparameters["popUpAlgoFollow"]:
+    from zebrazoom.code.popUpAlgoFollow import prepend
     prepend("Tracking done for well "+ str(wellNumber))
   
   if hyperparameters["detectMovementWithRawVideoInsideTracking"]:
