@@ -1,11 +1,13 @@
 import math
 import sys
 
+import numpy as np
+
 import cv2
 
 from PyQt5.QtCore import pyqtSignal, Qt, QAbstractAnimation, QEventLoop, QLine, QParallelAnimationGroup, QPoint, QPointF, QPropertyAnimation, QRectF, QSize, QSizeF, QStandardPaths, QTimer
 from PyQt5.QtGui import QBrush, QColor, QFont, QImage, QPainter, QPen, QPixmap, QPolygonF, QTransform
-from PyQt5.QtWidgets import QApplication, QFrame, QGraphicsPixmapItem, QGraphicsScene, QGraphicsView, QGridLayout, QLabel, QLayout, QHBoxLayout, QPushButton, QScrollArea, QSizePolicy, QSlider, QSpinBox, QSplitter, QSplitterHandle, QToolButton, QToolTip, QVBoxLayout, QWidget
+from PyQt5.QtWidgets import QApplication, QDoubleSpinBox, QFrame, QGraphicsPixmapItem, QGraphicsScene, QGraphicsView, QGridLayout, QLabel, QLayout, QHBoxLayout, QPushButton, QScrollArea, QSizePolicy, QSlider, QSpinBox, QSplitter, QSplitterHandle, QToolButton, QToolTip, QVBoxLayout, QWidget
 PYQT6 = False
 
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
@@ -28,14 +30,28 @@ LIGHT_CYAN = '#E0FFFF'
 LIGHT_GREEN = '#90ee90'
 GOLD = '#FFD700'
 SPINBOX_STYLESHEET = '''
-QSpinBox::down-button  {
+QSpinBox::down-button {
   subcontrol-origin: border;
   subcontrol-position: center left;
   height: 20;
   width: 20;
 }
 
-QSpinBox::up-button  {
+QSpinBox::up-button {
+  subcontrol-origin: border;
+  subcontrol-position: center right;
+  height: 20;
+  width: 20;
+}
+
+QDoubleSpinBox::down-button {
+  subcontrol-origin: border;
+  subcontrol-position: center left;
+  height: 20;
+  width: 20;
+}
+
+QDoubleSpinBox::up-button {
   subcontrol-origin: border;
   subcontrol-position: center right;
   height: 20;
@@ -107,7 +123,10 @@ def showBlockingPage(layout, title=None, buttons=(), dialog=False, labelInfo=Non
     signal.connect(lambda *args: loop.exit())
   mainLayout = QVBoxLayout()
   if title is not None:
-    mainLayout.addWidget(apply_style(QLabel(title), font=TITLE_FONT), alignment=Qt.AlignmentFlag.AlignCenter)
+    titleLabel = apply_style(QLabel(title), font=TITLE_FONT)
+    titleLabel.setMinimumSize(1, 1)
+    titleLabel.resizeEvent = lambda evt: titleLabel.setMinimumWidth(evt.size().width()) or titleLabel.setWordWrap(evt.size().width() <= titleLabel.sizeHint().width())
+    mainLayout.addWidget(titleLabel, alignment=Qt.AlignmentFlag.AlignCenter)
   mainLayout.addLayout(layout)
   mainLayout.addLayout(_getButtonsLayout(buttons, loop))
   app = QApplication.instance()
@@ -195,12 +214,9 @@ def _cvToPixmap(img):
   return QPixmap.fromImage(QImage(img.data.tobytes(), width, height, bytesPerLine, fmt))
 
 
-class ZoomableImage(QGraphicsView):
-  pointSelected = pyqtSignal(QPoint)
-  proceed = pyqtSignal()
-
+class _ZoomableImage(QGraphicsView):
   def __init__(self, parent=None):
-    super(ZoomableImage, self).__init__(parent)
+    super().__init__(parent)
     self._zoom = 0
     self._scene = QGraphicsScene(self)
     self._pixmap = QGraphicsPixmapItem()
@@ -211,7 +227,6 @@ class ZoomableImage(QGraphicsView):
     self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
     self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
     self.setFrameShape(QFrame.Shape.NoFrame)
-    self._point = None
     self._dragging = False
     self._tooltipShown = False
 
@@ -250,10 +265,7 @@ class ZoomableImage(QGraphicsView):
       self._update(0.8)
 
   def keyPressEvent(self, evt):
-    if self._point is not None and (evt.key() == Qt.Key.Key_Enter or evt.key() == Qt.Key.Key_Return):
-      self.proceed.emit()
-      return
-    elif evt.modifiers() & Qt.KeyboardModifier.ControlModifier:
+    if evt.modifiers() & Qt.KeyboardModifier.ControlModifier:
       if evt.key() == Qt.Key.Key_Plus:
         self._zoom += 1
         self._update(1.25)
@@ -270,19 +282,42 @@ class ZoomableImage(QGraphicsView):
       QApplication.setOverrideCursor(Qt.CursorShape.ClosedHandCursor)
     super().mouseMoveEvent(evt)
 
+  def enterEvent(self, evt):
+    QApplication.setOverrideCursor(Qt.CursorShape.CrossCursor)
+    super().enterEvent(evt)
+
+  def leaveEvent(self, evt):
+    QApplication.restoreOverrideCursor()
+    super().leaveEvent(evt)
+
+
+class _ZoomableImagePoint(_ZoomableImage):
+  pointSelected = pyqtSignal(QPoint)
+  proceed = pyqtSignal()
+
+  def __init__(self, parent=None):
+    super().__init__(parent)
+    self._point = None
+
+  def keyPressEvent(self, evt):
+    if self._point is not None and (evt.key() == Qt.Key.Key_Enter or evt.key() == Qt.Key.Key_Return):
+      self.proceed.emit()
+      return
+    super().keyPressEvent(evt)
+
   def mouseReleaseEvent(self, evt):
     if not self._dragging:
       if self._pixmap.isUnderMouse():
         self._point = self.mapToScene(evt.pos()).toPoint()
         self.viewport().update()
         self.pointSelected.emit(self._point)
-        if not self._tooltipShown:
+        if not self._tooltipShown and self._point is not None:
           QToolTip.showText(self.mapToGlobal(self._point), "If you aren't satisfied with the selection, click again.", self, self.rect(), 5000)
           self._tooltipShown = True
     else:
       self._dragging = False
       QApplication.restoreOverrideCursor()
-    super(ZoomableImage, self).mouseReleaseEvent(evt)
+    super().mouseReleaseEvent(evt)
 
   def paintEvent(self, evt):
     super().paintEvent(evt)
@@ -295,14 +330,67 @@ class ZoomableImage(QGraphicsView):
       qp.drawEllipse(self.mapFromScene(QPointF(self._point)), 2, 2)
     qp.end()
 
-  def enterEvent(self, evt):
-    QApplication.setOverrideCursor(Qt.CursorShape.CrossCursor)
-    super().enterEvent(evt)
+
+class _ZoomableImageCircle(_ZoomableImage):
+  circleSelected = pyqtSignal(bool)
+
+  def __init__(self):
+    super().__init__()
+    self._center = None
+    self._currentPosition = None
+    self._borderPoint = None
+    self.viewport().setMouseTracking(True)
+
+  def mouseReleaseEvent(self, evt):
+    if not self._dragging:
+      if self._pixmap.isUnderMouse():
+        if self._center is None or self._borderPoint is not None:
+          self._center = self.mapToScene(evt.pos()).toPoint()
+          self._borderPoint = None
+          self._currentPosition = None
+          self.circleSelected.emit(False)
+        else:
+          self._borderPoint = self.mapToScene(evt.pos()).toPoint()
+          self.circleSelected.emit(True)
+        self.viewport().update()
+        if not self._tooltipShown and self._center is not None and self._borderPoint is not None:
+          QToolTip.showText(self.mapToGlobal(self._center), "If you aren't satisfied with the selection, click again.", self, self.rect(), 5000)
+          self._tooltipShown = True
+    else:
+      self._dragging = False
+      QApplication.restoreOverrideCursor()
+    super().mouseReleaseEvent(evt)
+
+  def mouseMoveEvent(self, evt):
+    super().mouseMoveEvent(evt)
+    if not self._dragging:
+      self._currentPosition = self.mapToScene(evt.pos()).toPoint()
+      self.viewport().update()
 
   def leaveEvent(self, evt):
-    QApplication.restoreOverrideCursor()
+    self._currentPosition = None
+    self.viewport().update()
     super().leaveEvent(evt)
 
+  def paintEvent(self, evt):
+    super().paintEvent(evt)
+    if self._currentPosition is None and self._center is None:
+      return
+    qp = QPainter(self.viewport())
+    if self._center is not None:
+      qp.setBrush(QColor(255, 0, 0))
+      qp.setPen(Qt.PenStyle.NoPen)
+      borderPoint = self._borderPoint if self._borderPoint is not None else self._currentPosition if self._currentPosition is not None else None
+      mappedCenter = self.mapFromScene(self._center)
+      qp.drawEllipse(mappedCenter, 2, 2)
+      if borderPoint is not None:
+        delta = self.mapFromScene(borderPoint) - mappedCenter
+        radius = int(math.sqrt(delta.x() * delta.x() + delta.y() * delta.y()))
+        qp.setBrush(Qt.BrushStyle.NoBrush)
+        qp.setPen(QColor(0, 0, 255))
+        qp.drawEllipse(mappedCenter, radius, radius)
+    qp.end()
+
 
 def setPixmapFromCv(img, label, preferredSize=None, zoomable=False):
   originalPixmap = _cvToPixmap(img)
@@ -324,7 +412,7 @@ def setPixmapFromCv(img, label, preferredSize=None, zoomable=False):
     label.setPixmap(pixmap)
   else:
     label.hide()
-    image = ZoomableImage()
+    image = _ZoomableImagePoint() if not isinstance(label, _InteractiveLabelCircle) else _ZoomableImageCircle()
     image.sizeHint = lambda: size
     image.setMaximumSize(size)
     image.viewport().setFixedSize(size)
@@ -337,12 +425,45 @@ def setPixmapFromCv(img, label, preferredSize=None, zoomable=False):
         label.getCoordinates = lambda: (point.x(), point.y())
       image.pointSelected.connect(pointSelected)
       image.proceed.connect(label.proceed.emit)
+    if hasattr(image, "circleSelected"):
+      def circleSelected(selected):
+        label.circleSelected.emit(selected)
+      image.circleSelected.connect(circleSelected)
+      def calculateRadius():
+        if image._borderPoint is None or image._center is None:
+          return None, None
+        delta = image._borderPoint - image._center
+        return image._center, int(math.sqrt(delta.x() * delta.x() + delta.y() * delta.y()))
+      label.getInfo = calculateRadius
+
+
+class _DoubleSlider(QSlider):
+  _DECIMALS = 2
+
+  def __init__(self, *args, **kargs):
+    super().__init__( *args, **kargs)
+    self._factor = 10 ** self._DECIMALS
+
+  def value(self):
+    return super().value() / self._factor
+
+  def setMinimum(self, value):
+    return super().setMinimum(int(value * self._factor))
+
+  def setMaximum(self, value):
+    return super().setMaximum(int(value * self._factor))
+
+  def setRange(self, minimum, maximum):
+    return super().setRange(int(minimum * self._factor), int(maximum * self._factor))
+
+  def setValue(self, value):
+    super().setValue(int(value * self._factor))
 
 
 class SliderWithSpinbox(QWidget):
   valueChanged = pyqtSignal(int)
 
-  def __init__(self, value, minimum, maximum, name=None):
+  def __init__(self, value, minimum, maximum, name=None, double=False):
     super().__init__()
     minimum = int(minimum)
     maximum = int(maximum)
@@ -358,13 +479,13 @@ class SliderWithSpinbox(QWidget):
     layout.addWidget(minLabel, 1, 1, Qt.AlignmentFlag.AlignLeft)
     maxLabel = QLabel(str(maximum))
     layout.addWidget(maxLabel, 1, 3, Qt.AlignmentFlag.AlignRight)
-    slider = QSlider(Qt.Orientation.Horizontal)
+    slider = QSlider(Qt.Orientation.Horizontal) if not double else _DoubleSlider(Qt.Orientation.Horizontal)
     slider.setMinimumWidth(350)
     slider.setRange(minimum, maximum)
     slider.setValue(value)
     layout.addWidget(slider, 2, 1, 1, 3)
 
-    spinbox = QSpinBox()
+    spinbox = QSpinBox() if not double else QDoubleSpinBox()
     spinbox.setStyleSheet(SPINBOX_STYLESHEET)
     spinbox.setMinimumWidth(90)
     spinbox.setRange(minimum, maximum)
@@ -373,7 +494,9 @@ class SliderWithSpinbox(QWidget):
 
     def spinboxValueChanged():
       value = spinbox.value()
+      blocked = slider.blockSignals(True)
       slider.setValue(value)
+      slider.blockSignals(blocked)
       self.valueChanged.emit(value)
     spinbox.valueChanged.connect(spinboxValueChanged)
     slider.valueChanged.connect(lambda: spinbox.setValue(slider.value()))
@@ -396,6 +519,7 @@ class SliderWithSpinbox(QWidget):
     self.setMinimum = lambda min_: spinbox.setMinimum(min_) or slider.setMinimum(min_) or minLabel.setText(str(int(min_)))
     self.setMaximum = lambda max_: spinbox.setMaximum(max_) or slider.setMaximum(max_) or maxLabel.setText(str(int(max_)))
     self.setRange = lambda min_, max_: spinbox.setRange(min_, max_) or slider.setRange(min_, max_)
+    self.setSingleStep = lambda step: spinbox.setSingleStep(step) or slider.setSingleStep(step)
     self.isSliderDown = slider.isSliderDown
     self.sliderWidth = slider.width
     self.setPosition = slider.setSliderPosition
@@ -643,7 +767,15 @@ def getPoint(frame, title, extraButtons=(), selectingRegion=False, backBtnCb=Non
   layout.addWidget(QLabel(additionalText), alignment=Qt.AlignmentFlag.AlignCenter)
 
   video = _InteractiveLabelPoint(width, height, selectingRegion)
-  extraButtons = tuple((text, lambda: cb(video), exitLoop) for text, cb, exitLoop in extraButtons)
+
+  def callback(cb):
+    if zoomable:
+      zoomableImage = layout.itemAt(1).widget()
+      layout.replaceWidget(zoomableImage, video)
+      zoomableImage.hide()
+      video.show()
+    cb(video)
+  extraButtons = tuple((text, lambda: callback(cb), exitLoop) for text, cb, exitLoop in extraButtons)
   buttons = (("Back", backBtnCb, True),) if backBtnCb is not None else ()
   buttons += (("Next", None, True, video.pointSelected),) if useNext else ()
   layout.addWidget(video, alignment=Qt.AlignmentFlag.AlignCenter, stretch=1)
@@ -659,17 +791,25 @@ def getPoint(frame, title, extraButtons=(), selectingRegion=False, backBtnCb=Non
 class _InteractiveLabelRect(QLabel):
   regionSelected = pyqtSignal(bool)
 
-  def __init__(self, width, height):
+  def __init__(self, width, height, initialRect):
     super().__init__()
     self._width = width
     self._height = height
-    self._topLeft = None
     self._currentPosition = None
+    if initialRect is not None:
+      self._initialRect = initialRect
+    self._topLeft = None
     self._bottomRight = None
-    self._size = None
+    self._size = self.size()
     self._tooltipShown = False
     self.setMouseTracking(True)
 
+  def clearRectangle(self):
+    self._topLeft = None
+    self._bottomRight = None
+    self.update()
+    self.regionSelected.emit(False)
+
   def mousePressEvent(self, evt):
     if self._topLeft is None or self._bottomRight is not None:
       self._topLeft = evt.pos()
@@ -700,6 +840,9 @@ class _InteractiveLabelRect(QLabel):
 
   def paintEvent(self, evt):
     super().paintEvent(evt)
+    if hasattr(self, '_initialRect'):
+      self._topLeft, self._bottomRight = (transformCoordinates(QRectF(QPointF(0, 0), QSizeF(self._width, self._height)), QRectF(QPointF(0, 0), QSizeF(self.size())), point) for point in self._initialRect)
+      del self._initialRect
     if self._currentPosition is None and self._topLeft is None:
       return
     qp = QPainter()
@@ -724,6 +867,12 @@ class _InteractiveLabelRect(QLabel):
 
   def resizeEvent(self, evt):
     super().resizeEvent(evt)
+    if self._topLeft is not None:
+      self._topLeft = transformCoordinates(QRectF(QPointF(0, 0), QSizeF(self._size)), QRectF(QPointF(0, 0), QSizeF(evt.size())), self._topLeft)
+    if self._bottomRight is not None:
+      self._bottomRight = transformCoordinates(QRectF(QPointF(0, 0), QSizeF(self._size)), QRectF(QPointF(0, 0), QSizeF(evt.size())), self._bottomRight)
+    if self._currentPosition:
+      self._currentPosition = transformCoordinates(QRectF(QPointF(0, 0), QSizeF(self._size)), QRectF(QPointF(0, 0), QSizeF(evt.size())), self._currentPosition)
     self._size = self.size()
 
   def getCoordinates(self):
@@ -735,17 +884,30 @@ class _InteractiveLabelRect(QLabel):
     return ([point.x(), point.y()] for point in points)
 
 
-def getRectangle(frame, title, backBtnCb=None, dialog=False):
+def getRectangle(frame, title, backBtnCb=None, dialog=False, buttons=None, initialRect=None, allowEmpty=False, contrastCheckbox=None, getFrame=None):
   height, width, _ = frame.shape
 
   layout = QVBoxLayout()
 
-  video = _InteractiveLabelRect(width, height)
-  layout.addWidget(video, alignment=Qt.AlignmentFlag.AlignCenter)
-  if backBtnCb is not None:
-    buttons = (("Back", backBtnCb, True), ("Next", None, True, video.regionSelected))
+  video = _InteractiveLabelRect(width, height, initialRect)
+  layout.addWidget(video, alignment=Qt.AlignmentFlag.AlignCenter, stretch=1)
+  if contrastCheckbox is not None:
+    assert getFrame is not None
+    contrastCheckbox.toggled.connect(lambda checked: setPixmapFromCv(getFrame(checked), video))
+    layout.addWidget(contrastCheckbox, alignment=Qt.AlignmentFlag.AlignCenter)
+  if allowEmpty:
+    clearRectangleBtn = QPushButton('Clear rectangle')
+    clearRectangleBtn.clicked.connect(video.clearRectangle)
+    clearRectangleBtn.setEnabled(initialRect is not None)
+    layout.addWidget(clearRectangleBtn, alignment=Qt.AlignmentFlag.AlignCenter)
+    video.regionSelected.connect(clearRectangleBtn.setEnabled)
+  if buttons is None:
+    if backBtnCb is not None:
+      buttons = (("Back", backBtnCb, True), ("Next", None, True, video.regionSelected))
+    else:
+      buttons = (("Next", None, True, video.regionSelected),)
   else:
-    buttons = (("Next", None, True, video.regionSelected),)
+    assert backBtnCb is None
   if not dialog:
     showBlockingPage(layout, title=title, buttons=buttons, labelInfo=(frame, video))
   else:
@@ -910,7 +1072,7 @@ def addToHistory(fn):
 
 class Expander(QWidget):
   def __init__(self, parent, title, layout, animationDuration=200, showFrame=False, addScrollbars=False):
-    super(Expander, self).__init__()
+    super().__init__()
 
     self._toggleButton = toggleButton = QToolButton()
     toggleButton.setStyleSheet("QToolButton { border: none; }")
@@ -1054,7 +1216,7 @@ class _InteractiveLabelCircle(QLabel):
     return center, int(math.sqrt(radius.dx() * radius.dx() + radius.dy() * radius.dy()))
 
 
-def getCircle(frame, title, backBtnCb=None):
+def getCircle(frame, title, backBtnCb=None, zoomable=False):
   height, width, _ = frame.shape
 
   layout = QVBoxLayout()
@@ -1065,7 +1227,7 @@ def getCircle(frame, title, backBtnCb=None):
     buttons = (("Cancel", backBtnCb, True), ("Ok", None, True, video.circleSelected))
   else:
     buttons = (("Ok", None, True, video.circleSelected),)
-  showBlockingPage(layout, title=title, buttons=buttons, labelInfo=(frame, video))
+  showBlockingPage(layout, title=title, buttons=buttons, labelInfo=(frame, video, zoomable))
   return video.getInfo()
 
 
@@ -1124,3 +1286,21 @@ class CollapsibleSplitter(QSplitter):
 
     handle.setLayout(layout)
     return handle
+
+
+def improveContrast(frame, quartile):
+  frame = 255 - frame
+  frameIsBGR = len(frame.shape) == 3
+  if frameIsBGR:
+    frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
+  lowVal = int(np.quantile(frame, quartile))
+  highVal = int(np.quantile(frame, 1 - quartile))
+  frame[frame < lowVal] = lowVal
+  frame[frame > highVal] = highVal
+  frame = frame - lowVal
+  mult = np.max(frame)
+  frame = frame * (255 / mult)
+  frame = frame.astype('uint8')
+  if frameIsBGR:
+    frame = cv2.cvtColor(frame, cv2.COLOR_GRAY2BGR)
+  return frame
