@@ -1,3 +1,4 @@
+import zebrazoom
 from zebrazoom.code.findWells import findWells
 from zebrazoom.code.getBackground import getBackground
 from zebrazoom.code.getImage.getForegroundImage import getForegroundImage
@@ -6,9 +7,9 @@ from zebrazoom.code.extractParameters import extractParameters
 from zebrazoom.code.createSuperStruct import createSuperStruct
 from zebrazoom.code.createValidationVideo import createValidationVideo
 from zebrazoom.code.getHyperparameters import getHyperparameters
-import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
 from zebrazoom.code.dataPostProcessing.dataPostProcessing import dataPostProcessing
 from zebrazoom.code.fasterMultiprocessing import fasterMultiprocessing
+from zebrazoom.code.fasterMultiprocessing2 import fasterMultiprocessing2
 
 import sys
 import pickle
@@ -21,18 +22,19 @@ import json
 import subprocess
 import glob
 
-from PyQt5.QtWidgets import QApplication
-
 from zebrazoom.code.vars import getGlobalVariables
 globalVariables = getGlobalVariables()
 
 output = 0
 
 # Does the tracking and then the extraction of parameters
-def getParametersForWell(videoPath,background,wellNumber,wellPositions,output,previouslyAcquiredTrackingDataForDebug,hyperparameters, videoName, dlModel):
-  if QApplication.instance() is None:
-    from zebrazoom.GUIAllPy import PlainApplication
-    app = PlainApplication(sys.argv)
+def getParametersForWell(videoPath,background,wellNumber,wellPositions,output,previouslyAcquiredTrackingDataForDebug,hyperparameters, videoName, dlModel, useGUI):
+  if useGUI:
+    from PyQt5.QtWidgets import QApplication
+
+    if QApplication.instance() is None:
+      from zebrazoom.GUIAllPy import PlainApplication
+      app = PlainApplication(sys.argv)
   if hyperparameters["debugPauseBetweenTrackAndParamExtract"] == "noDebug":
     # Normal execution process
     trackingData = tracking(videoPath,background,wellNumber,wellPositions,hyperparameters, videoName, dlModel)
@@ -54,7 +56,7 @@ def getParametersForWell(videoPath,background,wellNumber,wellPositions,output,pr
     output.put([wellNumber,parameters,[]])
 
 
-def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
+def mainZZ(pathToVideo, videoName, videoExt, configFile, argv, useGUI=True):
   
   videoNameWithExt = videoName + '.' + videoExt
   previouslyAcquiredTrackingDataForDebug = []
@@ -103,6 +105,8 @@ def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
   
   # Launching GUI algoFollower if necessary
   if hyperparameters["popUpAlgoFollow"]:
+    import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
+
     popUpAlgoFollow.createTraceFile("starting ZebraZoom analysis on " + videoName)
     p = Process(target=popUpAlgoFollow.initialise)
     p.start()
@@ -148,7 +152,7 @@ def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
   
 
   # Getting well positions
-  if hyperparameters["headEmbeded"]:
+  if hyperparameters["headEmbeded"] and not hyperparameters["oneWellManuallyChosenTopLeft"]:
     wellPositions = [{"topLeftX":0, "topLeftY":0, "lengthX": hyperparameters["videoWidth"], "lengthY": hyperparameters["videoHeight"]}]
   else:
     print("start find wells")
@@ -174,20 +178,25 @@ def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
         pickle.dump(wellPositions, outfile2)
         outfile2.close()
     outfile.close()
-    app = QApplication.instance()
-    if hasattr(app, "wellPositions"):
-      if wellPositions is None:
-        return
-      app.wellPositions[:] = [(position['topLeftX'], position['topLeftY'], position['lengthX'], position['lengthY'])
-                              for idx, position in enumerate(wellPositions)]
-      if hyperparameters["wellsAreRectangles"] or len(hyperparameters["oneWellManuallyChosenTopLeft"]) or int(hyperparameters["multipleROIsDefinedDuringExecution"]) or hyperparameters["noWellDetection"] or hyperparameters["groupOfMultipleSameSizeAndShapeEquallySpacedWells"]:
-        app.wellShape = 'rectangle'
-      else:
-        app.wellShape = 'circle'
+    if useGUI:
+      from PyQt5.QtWidgets import QApplication
+
+      app = QApplication.instance()
+      if hasattr(app, "wellPositions"):
+        if wellPositions is None:
+          return
+        app.wellPositions[:] = [(position['topLeftX'], position['topLeftY'], position['lengthX'], position['lengthY'])
+                                for idx, position in enumerate(wellPositions)]
+        if hyperparameters["wellsAreRectangles"] or len(hyperparameters["oneWellManuallyChosenTopLeft"]) or int(hyperparameters["multipleROIsDefinedDuringExecution"]) or hyperparameters["noWellDetection"] or hyperparameters["groupOfMultipleSameSizeAndShapeEquallySpacedWells"]:
+          app.wellShape = 'rectangle'
+        else:
+          app.wellShape = 'circle'
 
   if int(hyperparameters["exitAfterWellsDetection"]):
     print("exitAfterWellsDetection")
     if hyperparameters["popUpAlgoFollow"]:
+      import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
+
       popUpAlgoFollow.prepend("ZebraZoom Analysis finished for " + videoName)
     raise ValueError
     
@@ -206,9 +215,12 @@ def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
       pickle.dump(background, outfile)
       cv2.imwrite(os.path.join(outputFolderVideo, 'background.png'), background)
     outfile.close()
-    app = QApplication.instance()
-    if hasattr(app, "background"):
-      app.background = background
+    if useGUI:
+      from PyQt5.QtWidgets import QApplication
+
+      app = QApplication.instance()
+      if hasattr(app, "background"):
+        app.background = background
   if hyperparameters["exitAfterBackgroundExtraction"]:
     print("exitAfterBackgroundExtraction")
     raise ValueError
@@ -224,27 +236,30 @@ def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
   if hyperparameters["fasterMultiprocessing"] == 1:
     processes = -1
     output2 = fasterMultiprocessing(os.path.join(pathToVideo, videoNameWithExt), background, wellPositions, [], hyperparameters, videoName)
+  elif hyperparameters["fasterMultiprocessing"] == 2:
+    processes = -1
+    output2 = fasterMultiprocessing2(os.path.join(pathToVideo, videoNameWithExt), background, wellPositions, [], hyperparameters, videoName)
   else:
     if globalVariables["noMultiprocessing"] == 0:
       if hyperparameters["onlyTrackThisOneWell"] == -1:
         # for all wells, in parallel
         processes = []
         for wellNumber in range(0,hyperparameters["nbWells"]):
-          p = Process(target=getParametersForWell, args=(os.path.join(pathToVideo, videoNameWithExt), background, wellNumber, wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel))
+          p = Process(target=getParametersForWell, args=(os.path.join(pathToVideo, videoNameWithExt), background, wellNumber, wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel, useGUI))
           p.start()
           processes.append(p)
       else:
         # for just one well
         processes = [1]
-        getParametersForWell(os.path.join(pathToVideo, videoNameWithExt), background, hyperparameters["onlyTrackThisOneWell"], wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel)
+        getParametersForWell(os.path.join(pathToVideo, videoNameWithExt), background, hyperparameters["onlyTrackThisOneWell"], wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel, useGUI)
     else:
       if hyperparameters["onlyTrackThisOneWell"] == -1:
         processes = [1 for i in range(0, hyperparameters["nbWells"])]
         for wellNumber in range(0,hyperparameters["nbWells"]):
-          getParametersForWell(os.path.join(pathToVideo, videoNameWithExt), background, wellNumber, wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel)
+          getParametersForWell(os.path.join(pathToVideo, videoNameWithExt), background, wellNumber, wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel, useGUI)
       else:
         processes = [1]
-        getParametersForWell(os.path.join(pathToVideo, videoNameWithExt), background, hyperparameters["onlyTrackThisOneWell"], wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel)
+        getParametersForWell(os.path.join(pathToVideo, videoNameWithExt), background, hyperparameters["onlyTrackThisOneWell"], wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel, useGUI)
   
   # Sorting wells after the end of the parallelized calls end
   if processes != -1:
@@ -298,7 +313,7 @@ def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
     
   try:
     with open(os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'ZebraZoomVersionUsed.txt'), 'w') as fp:
-      subprocess.run(["pip", "show",  "zebrazoom"], stdout=fp)
+      fp.write(zebrazoom.__version__)
   except:
     fileVersionUsed = open(os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'ZebraZoomVersionUsed.txt'), 'w')
     fileVersionUsed.write("Was not able to retrive the version number used.")
@@ -326,6 +341,8 @@ def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
   
   
   if hyperparameters["popUpAlgoFollow"]:
+    import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
+
     popUpAlgoFollow.prepend("ZebraZoom Analysis finished for " + videoName)
     # popUpAlgoFollow.prepend("")
     # if hyperparameters["closePopUpWindowAtTheEnd"]:
